@@ -13,7 +13,7 @@ BEGIN {
     use vars        qw( @EXPORT @ISA $VERSION );
     @EXPORT     =   qw( shell fetch get install );
     @ISA        =   qw( Exporter );
-    $VERSION    =   "0.050_04";     #have to hardcode or cpan.org gets unhappy
+    $VERSION    =   "0.056_01";     #have to hardcode or cpan.org gets unhappy
 }
 
 ### purely for backward compatibility, so we can call it from the commandline:
@@ -21,25 +21,25 @@ BEGIN {
 sub install {
     my $cpan = CPANPLUS::Backend->new;
     my $mod = shift or (
-                    error(loc("No module specified!")), return
+                    cp_error(loc("No module specified!")), return
                 );
 
     if ( ref $mod ) {
-        error( loc( "You passed an object. Use %1 for OO style interaction",
+        cp_error( loc( "You passed an object. Use %1 for OO style interaction",
                     'CPANPLUS::Backend' ));
         return;
 
     } else {
         my $obj = $cpan->module_tree($mod) or (
-                        error(loc("No such module '%1'", $mod)),
+                        cp_error(loc("No such module '%1'", $mod)),
                         return
                     );
 
         my $ok = $obj->install;
 
         $ok
-            ? msg(loc("Installing of %1 successful", $mod),1)
-            : msg(loc("Installing of %1 failed", $mod),1);
+            ? cp_msg(loc("Installing of %1 successful", $mod),1)
+            : cp_msg(loc("Installing of %1 failed", $mod),1);
 
         return $ok;
     }
@@ -50,25 +50,25 @@ sub fetch {
     my $cpan = CPANPLUS::Backend->new;
 
     my $mod = shift or (
-                    error(loc("No module specified!")), return
+                    cp_error(loc("No module specified!")), return
                 );
 
     if ( ref $mod ) {
-        error( loc( "You passed an object. Use %1 for OO style interaction",
+        cp_error( loc( "You passed an object. Use %1 for OO style interaction",
                     'CPANPLUS::Backend' ));
         return;
 
     } else {
         my $obj = $cpan->module_tree($mod) or (
-                        error(loc("No such module '%1'", $mod)),
+                        cp_error(loc("No such module '%1'", $mod)),
                         return
                     );
 
         my $ok = $obj->fetch( fetchdir => '.' );
 
         $ok
-            ? msg(loc("Fetching of %1 successful", $mod),1)
-            : msg(loc("Fetching of %1 failed", $mod),1);
+            ? cp_msg(loc("Fetching of %1 successful", $mod),1)
+            : cp_msg(loc("Fetching of %1 failed", $mod),1);
 
         return $ok;
     }
@@ -337,7 +337,7 @@ Jos Boumans E<lt>kane@cpan.orgE<gt>.
 =head1 COPYRIGHT
 
 The CPAN++ interface (of which this module is a part of) is
-copyright (c) 2001, 2002, 2003, 2004, Jos Boumans E<lt>kane@cpan.orgE<gt>.
+copyright (c) 2001, 2002, 2003, 2004, 2005 Jos Boumans E<lt>kane@cpan.orgE<gt>.
 All rights reserved.
 
 This library is free software;
