@@ -3,7 +3,7 @@ use strict;
 
 use CPANPLUS::inc;
 use CPANPLUS::Internals::Constants;
-use CPANPLUS::Error                 qw[error msg];
+use CPANPLUS::Error;
 
 use Log::Message;
 use Module::Load                qw[load];
@@ -21,7 +21,7 @@ $VERSION = $CPANPLUS::Internals::VERSION = $CPANPLUS::Internals::VERSION;
 ### The minimum required config version
 ### Update this if we have incompatible config changes
 #$MIN_CONFIG_VERSION = $VERSION;
-$MIN_CONFIG_VERSION = '0.050_03';
+$MIN_CONFIG_VERSION = '0.050_04';
 
 
 =pod
@@ -72,7 +72,7 @@ sub new {
     ### this code may change between releases, depending on
     ### compatibillity with previous versions.
     unless( $class->_config_version_sufficient ) {
-        error(loc(  "Your config is of version '%1' but '%2' requires ".
+        cp_error(loc(  "Your config is of version '%1' but '%2' requires ".
                     "a config of '%3' or higher. Your '%4' is of " .
                     "version '%5', but your config requires a version ".
                     "of '%6' or higher. You will need to reconfigure",
@@ -88,7 +88,7 @@ sub new {
                 }, $class;
 
     unless( $self->_load_args( options => \%hash ) ) {
-        error(loc(qq[Unable to initialize configuration!]));
+        cp_error(loc(qq[Unable to initialize configuration!]));
         return;
     }
 
@@ -117,14 +117,14 @@ sub _load_cpanplus_config {
 
     my $ok;
     $@
-        ? error( loc("Could not load your personal config: %1: %2",
+        ? cp_error( loc("Could not load your personal config: %1: %2",
                     $ENV{$env}, "$@"), "\n",
                 loc("Falling back to system-wide config."), "\n" )
         : ($ok = 1) if $tried;
 
     unless($ok) {
         eval { load CPANPLUS::Config };
-        error("$@"), return if $@;
+        cp_error("$@"), return if $@;
     }
 
     return 1;
@@ -251,6 +251,8 @@ sub save {
     my $time = gmtime;
 
     load Data::Dumper;
+    local $Data::Dumper::Sortkeys = 1; 
+    
     my $data = Data::Dumper->Dump([$self->conf], ['conf']);
 
     ## get rid of the bless'ing
@@ -284,7 +286,7 @@ ${is}cut
 
 package CPANPLUS::Config;
 
-\$VERSION = "$CPANPLUS::Internals::VERSION";
+\$VERSION = "$MIN_CONFIG_VERSION";
 
 \$MIN_CPANPLUS_VERSION = "$CPANPLUS::Config::MIN_CPANPLUS_VERSION";
 
@@ -309,7 +311,7 @@ _END_OF_CONFIG_
 
     my $fh = new FileHandle;
     $fh->open(">$file")
-        or (error(loc("Could not open '%1' for writing: %2", $file, $!)),
+        or (cp_error(loc("Could not open '%1' for writing: %2", $file, $!)),
             return );
 
     $fh->print($msg);
@@ -467,7 +469,7 @@ sub AUTOLOAD {
     my $conf = $self->conf;
 
     unless( scalar @_ ) {
-        error loc("No arguments provided!");
+        cp_error( loc("No arguments provided!") );
         return;
     }
 
@@ -482,7 +484,7 @@ sub AUTOLOAD {
     $type .= $field if $field;
 
     unless ( exists $conf->{$type} ) {
-        error loc("Invalid method type: '%1'", $name);
+        cp_error( loc("Invalid method type: '%1'", $name) );
         return;
     }
 
@@ -494,8 +496,14 @@ sub AUTOLOAD {
             if( exists $conf->{$type}->{$key} ) {
                 push @list, $conf->{$type}->{$key};
 
-            } else {
-                error loc(q[No such key '%1' in field '%2'], $key, $type);
+            ### XXX EU::AI compatibility hack to provide lookups like in
+            ### cpanplus 0.04x; we renamed ->_get_build('base') to
+            ### ->get_conf('base')
+            } elsif ( $type eq '_build' and $key eq 'base' ) {
+                return $self->get_conf($key);  
+                
+            } else {     
+                cp_error( loc(q[No such key '%1' in field '%2'], $key, $type) );
                 return;
             }
 
@@ -512,7 +520,7 @@ sub AUTOLOAD {
                 $conf->{$type}->{$key} = $val;
 
             } else {
-                error loc(q[No such key '%1' in field '%2'], $key, $type);
+                cp_error( loc(q[No such key '%1' in field '%2'], $key, $type) );
                 return;
             }
         }
@@ -526,7 +534,7 @@ sub AUTOLOAD {
         while( my($key,$val) = each %args ) {
 
             if( exists $conf->{$type}->{$key} ) {
-                error( loc( q[Key '%1' already exists for field '%2'],
+                cp_error( loc( q[Key '%1' already exists for field '%2'],
                             $key, $type));
                 return;
             } else {
@@ -536,7 +544,7 @@ sub AUTOLOAD {
         return 1;
     } else {
 
-        error loc(q[Unknown action '%1'], $action);
+        cp_error( loc(q[Unknown action '%1'], $action) );
         return;
     }
 }
@@ -564,8 +572,9 @@ terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<CPANPLUS::Backend>, L<CPANPLUS::Conf>
+L<CPANPLUS::Backend>, L<CPANPLUS::Configure::Setup>
 
+=cut
 
 # Local variables:
 # c-indentation-style: bsd
