@@ -2,12 +2,13 @@ package CPANPLUS::Configure::Setup;
 
 use strict;
 use vars    qw(@ISA);
-@ISA    =   qw[CPANPLUS::Internals::Utils];
+@ISA    =   qw[CPANPLUS::Internals::Utils CPANPLUS::Shell];
 
 use CPANPLUS::inc;
 use CPANPLUS::Internals::Utils;
 use CPANPLUS::Internals::Constants;
-use CPANPLUS::Error                 qw[msg error];
+use CPANPLUS::Shell                 ();
+use CPANPLUS::Error;
 
 use Config;
 use Term::UI;
@@ -99,9 +100,14 @@ sub init {
     my $self = shift;
     my $conf = $self->configure_object;
 
-    my $loc = $self->_save_where or return;
-    $self->location($loc);
-
+    ### in case we are totally supposed to Just Guess
+    {   local   $Term::UI::AUTOREPLY =
+                $Term::UI::AUTOREPLY || $ENV{PERL_MM_USE_DEFAULT} || 0;
+    
+        my $loc = $self->_save_where or return;
+        $self->location($loc);
+    }
+    
     my $manual = $self->_manual;
 
     $Term::UI::AUTOREPLY = 1 if $manual;
@@ -272,10 +278,15 @@ to point to the chosen location, so it can be found again.
 }
 
 sub _home_dir {
-    return  exists $ENV{APPDATA}        ? $ENV{APPDATA}     :
-            exists $ENV{HOME}           ? $ENV{HOME}        :
-            exists $ENV{USERPROFILE}    ? $ENV{USERPROFILE} :
-            exists $ENV{WINDIR}         ? $ENV{WINDIR}      :  cwd();
+    my @os_home_envs = qw( APPDATA HOME USERPROFILE WINDIR SYS$LOGIN );
+
+    for my $env ( @os_home_envs ) {
+        next unless exists $ENV{ $env };
+        next unless defined $ENV{ $env } && length $ENV{ $env };
+        return $ENV{ $env } if -d $ENV{ $env };
+    }
+
+    return cwd();
 }
 
 sub _issue_non_default_config_warning {
@@ -811,11 +822,15 @@ You can change this setting by making CPANPLUS prefer the use of
 certain binary programs if they are available.
 
 ");
-
-        my $type = 'prefer_bin';
+        
+        ### default to using binaries if we don't have compress::zlib only
+        ### -- it'll get very noisy otherwise
+        my $type    = 'prefer_bin';
+        my $default = eval { require Compress::Zlib; 1 } ? 'n' : 'y';
+        
         my $yn   = $term->ask_yn(
                         prompt  => loc("Should I prefer the use of binary programs?"),
-                        default => $self->_get( $type => 'n' ),
+                        default => $self->_get( $type => $default ),
                     );
 
         print $yn
@@ -934,22 +949,22 @@ Again, if you don't understand this question, just press ENTER.
     {
         print loc("
 Some modules provide both a Build.PL (Module::Build) and a Makefile.PL
-(ExtUtils::MakeMaker).
+(ExtUtils::MakeMaker).  By default, CPANPLUS prefers Makefile.PL.
+
+Module::Build support is not bundled standard with CPANPLUS, but 
+requires you to install 'CPANPLUS::Dist::Build' from CPAN.
 
 Although Module::Build is a pure perl solution, which means you will
 not need a 'make' binary, it does have some limitations. The most
 important is that CPANPLUS is unable to uninstall any modules installed
 by Module::Build.
 
-Right now, we suggest you use the Makefile.PL provided by
-ExtUtils::Makemaker if you have the choice, but it is up to you.
-
 Again, if you don't understand this question, just press ENTER.
 
 ");
         my $type = 'prefer_makefile';
         my $yn = $term->ask_yn(
-                    prompt  => loc("Prefer Makefile.PL?"),
+                    prompt  => loc("Prefer Makefile.PL over Build.PL?"),
                     default => $self->_get( $type => 1 ),
                  );
 
@@ -1110,6 +1125,8 @@ Otherwise, select ASK to have us ask your permission to install them.
                         default => $default,
                         choices => [@$map{sort keys %$map}],
                     );
+        print "\n";
+
         while( my($key,$val) = each %$map ) {
             next unless $val eq $reply;
 
@@ -1150,6 +1167,8 @@ CPANPLUS can do for you later);
                 : loc("I won't use the MD5 checksums");
 
         $conf->set_conf( $type => $yn );
+
+        print "\n\n";
     }
 
     {
@@ -1441,13 +1460,21 @@ are done.
             }
 
             CHOICE: {
+                
+                ### doesn't play nice with Term::UI :(
+                ### should make t::ui figure out pager opens
+                #$self->_pager_open;     # host lists might be long
+            
                 my @reply = $term->get_reply(
                                     prompt  => loc('Please pick a site: '),
-                                    choices => [sort(keys %map), qw|Custom View Up Quit|],
+                                    choices => [sort(keys %map), 
+                                                qw|Custom View Up Quit|],
                                     default => $default,
                                     multi   => 1,
                             );
                 print "\n\n";
+                #$self->_pager_close;
+    
 
                 goto COUNTRY    if grep { $_ eq 'Up' }      @reply;
                 goto CUSTOM     if grep { $_ eq 'Custom' }  @reply;
@@ -1833,6 +1860,8 @@ you made a typo you need to fix.
                     default => 'n'
                 );
 
+    print "\n";
+
     return 1 unless $yn;
 
     return !system("$editor $file");
