@@ -6,12 +6,14 @@ use vars    qw[@ISA $STATUS];
 
 use CPANPLUS::inc;
 use CPANPLUS::Internals::Constants;
+use CPANPLUS::Internals::Constants::Report;
 use CPANPLUS::Error;
 use FileHandle;
 use Cwd;
 
 use IPC::Cmd                    qw[run];
 use Params::Check               qw[check];
+use File::Basename              qw[dirname];
 use Module::Load::Conditional   qw[can_load check_install];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
@@ -76,6 +78,21 @@ BOOL indicating if the C<make> (or C<Build>) command was successful.
 BOOL indicating if the C<make test> (or C<Build test>) command was 
 successful.
 
+=item prepared ()
+
+BOOL indicating if the C<prepare> call exited succesfully
+This gets set after C<perl Makefile.PL>
+
+=item distdir ()
+
+Full path to the directory in which the C<prepare> call took place,
+set after a call to C<prepare>. 
+
+=item created ()
+
+BOOL indicating if the C<create> call exited succesfully. This gets
+set after C<make> and C<make test>.
+
 =item installed ()
 
 BOOL indicating if the module was installed. This gets set after
@@ -120,13 +137,13 @@ sub format_available {
   
     my $mod = "ExtUtils::MakeMaker";
     unless( can_load( modules => { $mod => 0.0 } ) ) {
-        error( loc( "You do not have '%1' -- '%2' not available",
+        cp_error( loc( "You do not have '%1' -- '%2' not available",
                     $mod, __PACKAGE__ ) ); 
         return;
     }
     
     unless( $conf->get_program('make') ) { 
-        error( loc( "You do not have '%1' -- '%2' not available",
+        cp_error( loc( "You do not have '%1' -- '%2' not available",
                     'make', __PACKAGE__ ) ); 
         return;
     }
@@ -148,14 +165,229 @@ sub init {
     my $status  = $dist->status;
    
     $status->mk_accessors(qw[makefile make test created installed uninstalled
-                            _create_args _install_args] );
+                             bin_make _prepare_args _create_args _install_args]
+                        );
     
     return 1;
 }    
 
+=pod $bool = $dist->prepare(....)
+
+=cut
+
+sub prepare {
+    ### just in case you already did a create call for this module object
+    ### just via a different dist object
+    my $dist = shift;
+    my $self = $dist->parent;
+    
+    ### we're also the cpan_dist, since we don't need to have anything
+    ### prepared 
+    $dist    = $self->status->dist_cpan if      $self->status->dist_cpan;     
+    $self->status->dist_cpan( $dist )   unless  $self->status->dist_cpan;    
+
+    my $cb   = $self->parent;
+    my $conf = $cb->configure_object;
+    my %hash = @_;
+
+    my $dir;
+    unless( $dir = $self->status->extract ) {
+        cp_error( loc( "No dir found to operate on!" ) );
+        return;
+    }
+    
+    my $args;
+    my( $force, $verbose, $perl, $mmflags );
+    {   local $Params::Check::ALLOW_UNKNOWN = 1;
+        my $tmpl = {
+            perl            => {    default => 
+                                        $conf->get_program('perl') || $^X, 
+                                    store => \$perl },
+            makemakerflags  => {    default =>
+                                        $conf->get_conf('makemakerflags'),
+                                    store => \$mmflags },                 
+            force           => {    default => $conf->get_conf('force'), 
+                                    store   => \$force },
+            verbose         => {    default => $conf->get_conf('verbose'), 
+                                    store   => \$verbose },
+        };                                            
+
+        $args = check( $tmpl, \%hash ) or return;
+    }
+    
+    ### maybe we already ran a create on this object? ###
+    return 1 if $dist->status->prepared && !$force;
+        
+    ### store the arguments, so ->install can use them in recursive loops ###
+    $dist->status->_prepare_args( $args );
+    
+    ### chdir to work directory ###
+    my $orig = cwd();
+    unless( $cb->_chdir( dir => $dir ) ) {
+        cp_error( loc( "Could not chdir to build directory '%1'", $dir ) );
+        return;
+    }
+    
+    my $fail; 
+    RUN: {
+        ### don't run 'perl makefile.pl' again if there's a makefile already 
+        if( -e MAKEFILE->() && (-M MAKEFILE->() < -M $dir) && !$force ) {
+            cp_msg(loc("'%1' already exists, not running '%2 %3' again ".
+                    " unless you force",
+                    MAKEFILE->(), $perl, MAKEFILE_PL->() ), $verbose );
+            
+        } else {
+            unless( -e MAKEFILE_PL->() ) {
+                cp_msg(loc("No '%1' found - attempting to generate one",
+                        MAKEFILE_PL->() ), $verbose );
+                        
+                $dist->write_makefile_pl( 
+                            verbose => $verbose, 
+                            force   => $force 
+                        );
+                
+                ### bail out if there's no makefile.pl ###
+                unless( -e MAKEFILE_PL->() ) {
+                    cp_error( loc( "Could not find '%1' - cannot continue", 
+                                MAKEFILE_PL->() ) );
+        
+                    ### mark that we screwed up ###
+                    $dist->status->makefile(0);
+                    $fail++; last RUN;
+                }
+            }    
+    
+            ### you can turn off running this verbose by changing
+            ### the config setting below, although it is really not
+            ### recommended
+            my $run_verbose = $verbose || 
+                              $conf->get_conf('allow_build_interactivity') ||
+                              0;
+    
+            ### this makes MakeMaker use defaults if possible, according
+            ### to schwern. See ticket 8047 for details.
+            local $ENV{PERL_MM_USE_DEFAULT} = 1 unless $run_verbose; 
+    
+            ### turn off our PERL5OPT so no modules from CPANPLUS::inc get
+            ### included in the makefile.pl -- it should build without
+            ### also, modules that run in taint mode break if we leave
+            ### our code ref in perl5opt
+            ### XXX we've removed the ENV settings from cp::inc, so only need
+            ### to reset the @INC
+            #local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt || ''; 
+    
+            ### make sure it's a string, so that mmflags that have more than
+            ### one key value pair are passed as is, rather than as:
+            ### perl Makefile.PL "key=val key=>val"
+            my $captured; my $makefile_pl = MAKEFILE_PL->();
+            
+            ### setting autoflush to true fixes issue from rt #8047
+            my $flush = OPT_AUTOFLUSH;
+            unless(scalar run(  command => "$perl $flush $makefile_pl $mmflags",
+                                buffer  => \$captured,
+                                verbose => $run_verbose, # may be interactive   
+            ) ) {
+                cp_error( loc( "Could not run '%1 %2': %3 -- cannot continue",
+                            $perl, MAKEFILE_PL->(), $captured ) );
+                
+                $dist->status->makefile(0);
+                $fail++; last RUN;
+            }
+
+            ### put the output on the stack, don't print it
+            cp_msg( $captured, 0 );
+        }
+        
+        ### if we got here, we managed to make a 'makefile' ###
+        $dist->status->makefile( MAKEFILE->($dir) );               
+        
+        ### start resolving prereqs ###
+        my $prereqs = $self->status->prereqs;
+        
+        ### a hashref of prereqs on success, undef on failure ###
+        $prereqs    ||= $dist->_find_prereqs( 
+                                    verbose => $verbose,
+                                    file    => $dist->status->makefile 
+                                );
+        
+        unless( $prereqs ) {
+            cp_error( loc( "Unable to scan '%1' for prereqs", 
+                        $dist->status->makefile ) );
+
+            $fail++; last RUN;
+        }
+    }
+   
+	unless( $cb->_chdir( dir => $orig ) ) {
+        cp_error( loc( "Could not chdir back to start dir '%1'", $orig ) );
+    }   
+   
+    ### save where we wrote this stuff -- same as extract dir in normal
+    ### installer circumstances
+    $dist->status->distdir( $self->status->extract );
+   
+    return $dist->status->prepared( $fail ? 0 : 1);
+}
+
 =pod
 
-=head2 $bool = $dist->create([perl => '/path/to/perl', makemakerflags => 'EXTRA=FLAGS', make => '/path/to/make', makeflags => 'EXTRA=FLAGS', prereq_target => TARGET, skiptest => BOOL, force => BOOL, verbose => BOOL])
+=head2 $href = $dist->_find_prereqs( file => '/path/to/Makefile', [verbose => BOOL])
+
+Parses a C<Makefile> for C<PREREQ_PM> entries and distills from that
+any prerequisites mentioned in the C<Makefile>
+
+Returns a hash with module-version pairs on success and false on
+failure.
+
+=cut
+
+sub _find_prereqs {
+    my $dist = shift;
+    my $self = $dist->parent;
+    my $cb   = $self->parent;
+    my $conf = $cb->configure_object;
+    my %hash = @_;
+
+    my ($verbose, $file);
+    my $tmpl = {
+        verbose => { default => $conf->get_conf('verbose'), store => \$verbose },
+        file    => { required => 1, allow => FILE_READABLE, store => \$file },
+    };
+    
+    my $args = check( $tmpl, \%hash ) or return;      
+    
+    my $fh = FileHandle->new();
+    unless( $fh->open( $file ) ) {
+        cp_error( loc( "Cannot open '%1': %2", $file, $! ) );
+        return;
+    }
+    
+    my %p;
+    while( <$fh> ) {
+        my ($found) = m|^[\#]\s+PREREQ_PM\s+=>\s+(.+)|;         
+        
+        next unless $found;
+        
+        while( $found =~ m/(?:\s)([\w\:]+)=>(?:q\[(.*?)\],?|undef)/g ) {
+            if( defined $p{$1} ) {
+                cp_msg(loc("Warning: PREREQ_PM mentions '%1' more than once. " .
+                        "Last mention wins.", $1 ), $verbose );
+            }
+            
+            $p{$1} = $cb->_version_to_number(version => $2);                  
+        }
+        last;
+    }
+
+    $self->status->prereqs( \%p );
+    
+    ### just to make sure it's not the same reference ###
+    return { %p };                              
+}     
+
+=pod
+
+=head2 $bool = $dist->create([perl => '/path/to/perl', make => '/path/to/make', makeflags => 'EXTRA=FLAGS', prereq_target => TARGET, skiptest => BOOL, force => BOOL, verbose => BOOL])
 
 C<create> preps a distribution for installation. This means it will 
 run C<perl Makefile.PL>, C<make> and C<make test>. 
@@ -192,21 +424,18 @@ sub create {
 
     my $dir;
     unless( $dir = $self->status->extract ) {
-        error( loc( "No dir found to operate on!" ) );
+        cp_error( loc( "No dir found to operate on!" ) );
         return;
     }
     
     my $args;
     my( $force, $verbose, $make, $makeflags, $skiptest, $prereq_target, $perl, 
-        $mmflags, $prereq_format);
+        $mmflags, $prereq_format, $prereq_build);
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
         my $tmpl = {
             perl            => {    default => 
                                         $conf->get_program('perl') || $^X, 
                                     store => \$perl },
-            makemakerflags  => {    default =>
-                                        $conf->get_conf('makemakerflags'),
-                                    store => \$mmflags },                 
             force           => {    default => $conf->get_conf('force'), 
                                     store   => \$force },
             verbose         => {    default => $conf->get_conf('verbose'), 
@@ -221,7 +450,8 @@ sub create {
             ### don't set the default prereq format to 'makemaker' -- wrong!
             prereq_format   => {    #default => $self->status->installer_type,
                                     default => '',
-                                    store   => \$prereq_format },                    
+                                    store   => \$prereq_format },   
+            prereq_build    => {    default => 0, store => \$prereq_build },                                    
         };                                            
 
         $args = check( $tmpl, \%hash ) or return;
@@ -233,127 +463,54 @@ sub create {
     ### store the arguments, so ->install can use them in recursive loops ###
     $dist->status->_create_args( $args );
     
-    ### chdir to work directory ###
-    my $orig = cwd();
-    unless( $cb->_chdir( dir => $dir ) ) {
-        error( loc( "Could not chdir to build directory '%1'", $dir ) );
+    unless( $dist->status->prepared ) {
+        cp_error( loc( "You have not successfully prepared a '%2' distribution ".
+                    "yet -- cannot create yet", __PACKAGE__ ) );
         return;
     }
     
-    my $fail; my $prereq_fail;
+    
+    ### chdir to work directory ###
+    my $orig = cwd();
+    unless( $cb->_chdir( dir => $dir ) ) {
+        cp_error( loc( "Could not chdir to build directory '%1'", $dir ) );
+        return;
+    }
+    
+    my $fail; my $prereq_fail; 
     RUN: {
-        ### don't run 'perl makefile.pl' again if there's a makefile already ###
-        if( -e MAKEFILE->() && (-M MAKEFILE->() < -M $dir) && !$force ) {
-            msg(loc("'%1' already exists, not running '%2 %3' again ".
-                    " unless you force",
-                    MAKEFILE->(), $perl, MAKEFILE_PL->() ), $verbose );
-            
-        } else {
-            unless( -e MAKEFILE_PL->() ) {
-                msg(loc("No '%1' found - attempting to generate one",
-                        MAKEFILE_PL->() ), $verbose );
-                        
-                $dist->write_makefile_pl( 
-                            verbose => $verbose, 
-                            force   => $force 
-                        );
-                
-                ### bail out if there's no makefile.pl ###
-                unless( -e MAKEFILE_PL->() ) {
-                    error( loc( "Could not find '%1' - cannot continue", 
-                                MAKEFILE_PL->() ) );
+        ### this will set the directory back to the start
+        ### dir, so we must chdir /again/           
+        my $ok = $dist->_resolve_prereqs(
+                            format          => $prereq_format,
+                            verbose         => $verbose,
+                            prereqs         => $self->status->prereqs,
+                            target          => $prereq_target,
+                            force           => $force,
+                            prereq_build    => $prereq_build,
+                    );
         
-                    ### mark that we screwed up ###
-                    $dist->status->makefile(0);
-                    $fail++; last RUN;
-                }
-            }    
-    
-            ### you can turn off running this verbose by changing
-            ### the config setting below, although it is really not
-            ### recommended
-            my $run_verbose = $verbose || 
-                              $conf->get_conf('allow_build_interactivity') ||
-                              0;
-    
-            ### this makes MakeMaker use defaults if possible, according
-            ### to schwern. See ticket 8047 for details.
-            local $ENV{PERL_MM_USE_DEFAULT} = 1 unless $run_verbose; 
-    
-            ### turn off our PERL5OPT so no modules from CPANPLUS::inc get
-            ### included in the makefile.pl -- it should build without
-            ### also, modules that run in taint mode break if we leave
-            ### our code ref in perl5opt
-            ### XXX we've removed the ENV settings from cp::inc, so only need
-            ### to reset the @INC
-            #local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt || ''; 
-    
-            ### make sure it's a string, so that mmflags that have more than
-            ### one key value pair are passed as is, rather than as:
-            ### perl Makefile.PL "key=val key=>val"
-            my $captured; my $makefile_pl = MAKEFILE_PL->();
-            unless(scalar run(  command => "$perl $makefile_pl $mmflags",
-                                buffer  => \$captured,
-                                verbose => $run_verbose, # may be interactive   
-            ) ) {
-                error( loc( "Could not run '%1 %2': %3 -- cannot continue",
-                            $perl, MAKEFILE_PL->(), $captured ) );
-                
-                $dist->status->makefile(0);
-                $fail++; last RUN;
-            }
-        }
-        
-        ### if we got here, we managed to make a 'makefile' ###
-        $dist->status->makefile( MAKEFILE->($dir) );               
-        
-        ### start resolving prereqs ###
-        my $prereqs = $self->status->prereqs;
-        
-        ### a hashref of prereqs on success, undef on failure ###
-        $prereqs    ||= $dist->_find_prereqs( 
-                                    verbose => $verbose,
-                                    file    => $dist->status->makefile 
-                                );
-        
-        unless( $prereqs ) {
-            error( loc( "Unable to scan '%1' for prereqs", 
-                        $dist->status->makefile ) );
-        } else {
-            $self->status->prereqs( $prereqs ); 
-        
-            ### this will set the directory back to the start
-            ### dir, so we must chdir /again/           
-            my $ok = $dist->_resolve_prereqs(
-                                format   => $prereq_format,
-                                verbose  => $verbose,
-                                prereqs  => $prereqs,
-                                target   => $prereq_target
-                        
-                        );
-            
-            unless( $cb->_chdir( dir => $dir ) ) {
-                error( loc( "Could not chdir to build directory '%1'", $dir ) );
-                return;
-            }       
-                      
-            unless( $ok ) {
-           
-                #### use $dist->flush to reset the cache ###
-                error( loc( "Unable to satisfy prerequisites for '%1' " .
-                            "-- aborting install", $self->module ) );    
-                $dist->status->make(0);
-                $fail++; $prereq_fail++;
-                last RUN;
-            } 
-        }      
+        unless( $cb->_chdir( dir => $dir ) ) {
+            cp_error( loc( "Could not chdir to build directory '%1'", $dir ) );
+            return;
+        }       
+                  
+        unless( $ok ) {
+       
+            #### use $dist->flush to reset the cache ###
+            cp_error( loc( "Unable to satisfy prerequisites for '%1' " .
+                        "-- aborting install", $self->module ) );    
+            $dist->status->make(0);
+            $fail++; $prereq_fail++;
+            last RUN;
+        } 
         ### end of prereq resolving ###    
         
         my $captured;
         
         ### 'make' section ###    
         if( -d BLIB->($dir) && (-M BLIB->($dir) < -M $dir) && !$force ) {
-            msg(loc("Already ran '%1' for this module [%2] -- " .
+            cp_msg(loc("Already ran '%1' for this module [%2] -- " .
                     "not running again unless you force", 
                     $make, $self->module ), $verbose );
         } else {
@@ -361,11 +518,14 @@ sub create {
                                 buffer  => \$captured,
                                 verbose => $verbose ) 
             ) {
-                error( loc( "MAKE failed: %1 %2", $!, $captured ) );
+                cp_error( loc( "MAKE failed: %1 %2", $!, $captured ) );
                 $dist->status->make(0);
                 $fail++; last RUN;
             }
             
+            ### put the output on the stack, don't print it
+            cp_msg( $captured, 0 );
+
             $dist->status->make(1);
 
             ### add this directory to your lib ###
@@ -373,7 +533,8 @@ sub create {
                 directories => [ BLIB_LIBDIR->( $self->status->extract ) ]
             );
             
-            last RUN if $skiptest;
+            ### dont bail out here, there's a conditional later on
+            #last RUN if $skiptest;
         }
         
         ### 'make test' section ###                                           
@@ -384,7 +545,7 @@ sub create {
             ### also, modules that run in taint mode break if we leave
             ### our code ref in perl5opt
             local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt || '';
-        
+
             ### you can turn off running this verbose by changing
             ### the config setting below, although it is really not 
             ### recommended
@@ -395,11 +556,22 @@ sub create {
 
             ### XXX need to add makeflags here too? 
             ### yes, but they should really be split out -- see bug #4143
-            unless(run( command => [$make, 'test', $makeflags],
+            if(run( command => [$make, 'test', $makeflags],
                         buffer  => \$captured,
                         verbose => $run_verbose,
             ) ) {
-                error( loc( "MAKE TEST failed: %1 %2", $!, $captured ) );
+                ### tests might pass because it doesn't have any tests defined
+                ### log this occasion non-verbosely, so our test reporter can
+                ### pick up on this
+                if ( NO_TESTS_DEFINED->( $captured ) ) {
+                    cp_msg( NO_TESTS_DEFINED->( $captured ), 0 )
+                } else {
+                    cp_msg( loc( "MAKE TEST passed: %2", $captured ) );
+                }
+            
+                $dist->status->test(1);
+            } else {
+                cp_error( loc( "MAKE TEST failed: %1 %2", $!, $captured ) );
             
                 ### send out error report here? or do so at a higher level?
                 ### --higher level --kane.
@@ -408,15 +580,12 @@ sub create {
                 unless( $force ) {
                     $fail++; last RUN;     
                 }
-            
-            } else {
-                $dist->status->test(1);
             }
         }
     } #</RUN>
       
     unless( $cb->_chdir( dir => $orig ) ) {
-        error( loc( "Could not chdir back to start dir '%1'", $orig ) );
+        cp_error( loc( "Could not chdir back to start dir '%1'", $orig ) );
     }  
     
     ### send out test report?
@@ -428,7 +597,7 @@ sub create {
             buffer  => CPANPLUS::Error->stack_as_string,
             verbose => $verbose,
             force   => $force,
-        ) or error(loc("Failed to send test report for '%1'",
+        ) or cp_error(loc("Failed to send test report for '%1'",
                     $self->module ) );
     }            
             
@@ -437,62 +606,6 @@ sub create {
 
 =pod
 
-=head2 $href = $dist->_find_prereqs( file => '/path/to/Makefile', [verbose => BOOL])
-
-Parses a C<Makefile> for C<PREREQ_PM> entries and distills from that
-any prerequisites mentioned in the C<Makefile>
-
-Returns a hash with module-version pairs on success and false on
-failure.
-
-=cut
-
-sub _find_prereqs {
-    my $dist = shift;
-    my $self = $dist->parent;
-    my $cb   = $self->parent;
-    my $conf = $cb->configure_object;
-    my %hash = @_;
-
-    my ($verbose, $file);
-    my $tmpl = {
-        verbose => { default => $conf->get_conf('verbose'), store => \$verbose },
-        file    => { required => 1, allow => FILE_READABLE, store => \$file },
-    };
-    
-    my $args = check( $tmpl, \%hash ) or return;      
-    
-    my $fh = FileHandle->new();
-    unless( $fh->open( $file ) ) {
-        error( loc( "Cannot open '%1': %2", $file, $! ) );
-        return;
-    }
-    
-    my %p;
-    while( <$fh> ) {
-        my ($found) = m|^[\#]\s+PREREQ_PM\s+=>\s+(.+)|;         
-        
-        next unless $found;
-        
-        while( $found =~ m/(?:\s)([\w\:]+)=>(?:q\[(.*?)\],?|undef)/g ) {
-            if( defined $p{$1} ) {
-                msg(loc("Warning: PREREQ_PM mentions '%1' more than once. " .
-                        "Last mention wins.", $1 ), $verbose );
-            }
-            
-            $p{$1} = $cb->_version_to_number(version => $2);                  
-        }
-        last;
-    }
-
-    $self->status->prereqs( \%p );
-    
-    ### just to make sure it's not the same reference ###
-    return { %p };                              
-}       
-
-=pod
-
 =head2 $bool = $dist->install([make => '/path/to/make',  makemakerflags => 'EXTRA=FLAGS', force => BOOL, verbose => BOOL])
 
 C<install> runs the following command:
@@ -515,14 +628,14 @@ sub install {
     
     
     unless( $dist->status->created ) {
-        error( loc( "You have not successfully created a '%2' distribution yet " .
+        cp_error( loc( "You have not successfully created a '%2' distribution yet " .
                     "-- cannot install yet", __PACKAGE__ ) );
         return;
     }
  
     my $dir;
     unless( $dir = $self->status->extract ) {
-        error( loc( "No dir found to operate on!" ) );
+        cp_error( loc( "No dir found to operate on!" ) );
         return;
     }
     
@@ -547,7 +660,7 @@ sub install {
     if( defined $self->status->installed && 
         !$self->status->installed && !$force 
     ) {
-        error( loc( "Module '%1' has failed to install before this session " .
+        cp_error( loc( "Module '%1' has failed to install before this session " .
                     "-- aborting install", $self->module ) );
         return;
     }
@@ -557,7 +670,7 @@ sub install {
     
     my $orig = cwd();
     unless( $cb->_chdir( dir => $dir ) ) {
-        error( loc( "Could not chdir to build directory '%1'", $dir ) );
+        cp_error( loc( "Could not chdir to build directory '%1'", $dir ) );
         return;
     }
     
@@ -568,18 +681,21 @@ sub install {
     ### yes, but they should really be split out.. see bug #4143
     my $cmd     = [$make, 'install', $makeflags];
     my $sudo    = $conf->get_program('sudo');
-    unshift @$cmd, $sudo if $sudo;
+    unshift @$cmd, $sudo if $sudo and $>;
     
     unless(scalar run(  command => $cmd,
                         verbose => $verbose,
                         buffer  => \$captured,
     ) ) {                   
-        error( loc( "MAKE INSTALL failed: %1 %2", $!, $captured ) );
+        cp_error( loc( "MAKE INSTALL failed: %1 %2", $!, $captured ) );
         $fail++; 
     }       
+
+    ### put the output on the stack, don't print it
+    cp_msg( $captured, 0 );
     
     unless( $cb->_chdir( dir => $orig ) ) {
-        error( loc( "Could not chdir back to start dir '%1'", $orig ) );
+        cp_error( loc( "Could not chdir back to start dir '%1'", $orig ) );
     }   
     
     return $dist->status->installed( $fail ? 0 : 1 );
@@ -615,7 +731,7 @@ sub write_makefile_pl {
 
     my $dir;
     unless( $dir = $self->status->extract ) {
-        error( loc( "No dir found to operate on!" ) );
+        cp_error( loc( "No dir found to operate on!" ) );
         return;
     }
     
@@ -631,7 +747,7 @@ sub write_makefile_pl {
     
     my $file = MAKEFILE_PL->($dir);
     if( -s $file && !$force ) {
-        msg(loc("Already created '%1' - not doing so again without force", 
+        cp_msg(loc("Already created '%1' - not doing so again without force", 
                 $file ), $verbose );
         return 1;
     }     
@@ -645,7 +761,7 @@ sub write_makefile_pl {
 
     my $fh = new FileHandle;
     unless( $fh->open( ">$file" ) ) {
-        error( loc( "Could not create file '%1': %2", $file, $! ) );
+        cp_error( loc( "Could not create file '%1': %2", $file, $! ) );
         return;
     }
     
@@ -678,6 +794,78 @@ $prereqs
     return 1;
 }                         
         
+sub dist_dir {
+    ### just in case you already did a call for this module object
+    ### just via a different dist object
+    my $dist = shift;
+    my $self = $dist->parent;
+    $dist    = $self->status->dist_cpan if      $self->status->dist_cpan;     
+    $self->status->dist_cpan( $dist )   unless  $self->status->dist_cpan;    
+ 
+    my $cb   = $self->parent;
+    my $conf = $cb->configure_object;
+    my %hash = @_;
+    
+    my $make; my $verbose;
+    {   local $Params::Check::ALLOW_UNKNOWN = 1;
+        my $tmpl = {
+            make    => {    default => $conf->get_program('make'),
+                                    store => \$make },                 
+            verbose => {    default => $conf->get_conf('verbose'), 
+                                    store   => \$verbose },
+        };  
+    
+        check( $tmpl, \%hash ) or return;    
+    }
+
+
+    my $dir;
+    unless( $dir = $self->status->extract ) {
+        cp_error( loc( "No dir found to operate on!" ) );
+        return;
+    }
+    
+    ### chdir to work directory ###
+    my $orig = cwd();
+    unless( $cb->_chdir( dir => $dir ) ) {
+        cp_error( loc( "Could not chdir to build directory '%1'", $dir ) );
+        return;
+    }
+
+    my $fail; my $distdir;
+    TRY: {    
+        $dist->prepare( @_ ) or (++$fail, last TRY);
+
+
+        my $captured;             
+            unless(scalar run(  command => [$make, 'distdir'],
+                            buffer  => \$captured,
+                            verbose => $verbose ) 
+        ) {
+            cp_error( loc( "MAKE DISTDIR failed: %1 %2", $!, $captured ) );
+            ++$fail, last TRY;
+        }
+
+        ### /path/to/Foo-Bar-1.2/Foo-Bar-1.2
+        $distdir = File::Spec->catdir( $dir, $self->package_name . '-' .
+                                                $self->package_version );
+
+        unless( -d $distdir ) {
+            cp_error(loc("Do not know where '%1' got created", 'distdir'));
+            ++$fail, last TRY;
+        }
+    }
+
+    unless( $cb->_chdir( dir => $orig ) ) {
+        cp_error( loc( "Could not chdir to start directory '%1'", $orig ) );
+        return;
+    }
+
+    return if $fail;
+    return $distdir;
+}    
+
+
 1;
 
 # Local variables:
