@@ -40,7 +40,7 @@ otherwise.
 
     my $send_list = {
         %$query_list,
-        'Test::Reporter' => 1.21,
+        'Test::Reporter' => 1.27,
     };
 
     sub _have_query_report_modules {
@@ -146,12 +146,12 @@ sub _query_report {
     my $url = TESTERS_URL->($mod->package_name);
     my $req = HTTP::Request->new( GET => $url);
 
-    msg( loc("Fetching: '%1'", $url), $verbose );
+    cp_msg( loc("Fetching: '%1'", $url), $verbose );
 
     my $res = $ua->request( $req );
 
     unless( $res->is_success ) {
-        error( loc( "Fetching report for '%1' failed: %2",
+        cp_error( loc( "Fetching report for '%1' failed: %2",
                     $url, $res->message ) );
         return;
     }
@@ -162,7 +162,8 @@ sub _query_report {
 
     my @rv;
     for my $href ( @$aref ) {
-        next unless $all or $href->{'distversion'} eq $dist;
+        next unless $all or defined $href->{'distversion'} && 
+                            $href->{'distversion'} eq $dist;
 
         push @rv, { platform    => $href->{'platform'},
                     grade       => $href->{'action'},
@@ -251,14 +252,15 @@ sub _send_report {
 
     ### do you even /have/ test::reporter? ###
     unless( $self->_have_send_report_modules(verbose => 1) ) {
-        error( loc( "You don't have '%1' (or modules required by '%2') ".
+        cp_error( loc( "You don't have '%1' (or modules required by '%2') ".
                     "installed, you cannot report test results.",
                     'Test::Reporter', 'Test::Reporter' ) );
         return;
     }
 
     ### check arguments ###
-    my ($buffer, $failed, $mod, $verbose, $force, $address, $save, $dontcc);
+    my ($buffer, $failed, $mod, $verbose, $force, $address, $save, $dontcc,
+        $tests_skipped );
     my $tmpl = {
             module  => { required => 1, store => \$mod, allow => IS_MODOBJ },
             buffer  => { required => 1, store => \$buffer },
@@ -270,6 +272,8 @@ sub _send_report {
                             store => \$verbose },
             force   => { default  => $conf->get_conf('force'),
                             store => \$force },
+            tests_skipped   
+                    => { default => 0, store => \$tests_skipped },
     };
 
     check( $tmpl, \%hash ) or return;
@@ -287,33 +291,59 @@ sub _send_report {
 
     my $grade;
     ### check if this is a platform specific module ###
-    unless( RELEVANT_TEST_RESULT->($mod) ) {
-        msg(loc("'%1' is a platform specific module, and the test results on".
+    ### if we failed the test, there may be reasons why 
+    ### an 'NA' might have to be insted
+    if ( $failed ) {
+        unless( RELEVANT_TEST_RESULT->($mod) ) {
+            cp_msg(loc(
+                "'%1' is a platform specific module, and the test results on".
                 " your platform are not relevant --sending N/A grade.",
                 $name), $verbose);
+        
+            $grade = GRADE_NA;
+        
+        } elsif ( UNSUPPORTED_OS->( $buffer ) ) {
+            cp_msg(loc(
+                "'%1' is a platform specific module, and the test results on".
+                " your platform are not relevant --sending N/A grade.",
+                $name), $verbose);
+        
+            $grade = GRADE_NA;
+        
+        ### you dont have a high enough perl version?    
+        } elsif ( PERL_VERSION_TOO_LOW->( $buffer ) ) {
+            cp_msg(loc("'%1' requires a higher version of perl than your current ".
+                    "version -- sending N/A grade.", $name), $verbose);
+        
+            $grade = GRADE_NA;                
 
-        $grade = GRADE_NA;
+        ### perhaps where were no tests...
+        ### see if the thing even had tests ###
+        } elsif ( NO_TESTS_DEFINED->( $buffer ) ) {
+            $grade = GRADE_UNKNOWN;
 
-    ### you dont have a high enough perl version?    
-    } elsif ( PERL_VERSION_TOO_LOW->( $buffer ) ) {
-        msg(loc("'%1' requires a higher version of perl than your current ".
-                "version -- sending N/A grade.", $name), $verbose);
+        } else {
+            
+            $grade = GRADE_FAIL;
+        }
 
-        $grade = GRADE_NA;                
-
-    ### see if the thing even had tests ###
-    } elsif ( NO_TESTS_DEFINED->( $buffer ) ) {
-        $grade = GRADE_UNKNOWN;
-
-    ### see if it was a pass or fail ###
+    ### if we got here, it didn't fail and tests were present.. so a PASS
+    ### is in order
     } else {
-        $grade = $failed ? GRADE_FAIL : GRADE_PASS;
+        $grade = GRADE_PASS;
     }
 
     ### so an error occurred, let's see what stage it went wrong in ###
     my $message;
     if( $grade eq GRADE_FAIL or $grade eq GRADE_UNKNOWN) {
 
+        ### return if one or more missing external libraries
+        if( my @missing = MISSING_EXTLIBS_LIST->($buffer) ) {
+            cp_msg(loc("Not sending test report - " .
+                    "external libraries not pre-installed"));
+            return 1;
+        }
+
         ### will be 'fetch', 'make', 'test', 'install', etc ###
         my $stage   = TEST_FAIL_STAGE->($buffer);
 
@@ -322,22 +352,40 @@ sub _send_report {
                     and ($stage !~ /\btest\b/);
 
         ### the header
-        $message =  REPORT_MESSAGE_HEADER->( $author );
+        $message =  REPORT_MESSAGE_HEADER->( $int_ver, $author );
 
         ### the bit where we inform what went wrong
-        $message .= REPORT_MESSAGE_FAIL_HEADER->(
-                        $int_ver, $stage, $buffer );
+        $message .= REPORT_MESSAGE_FAIL_HEADER->( $stage, $buffer );
 
         ### was it missing prereqs? ###
         if( my @missing = MISSING_PREREQS_LIST->($buffer) ) {
-            $message .= REPORT_MISSING_PREREQS->(@missing);
+            if(!$self->_verify_missing_prereqs(
+                                module  => $mod,
+                                missing => \@missing
+                        )) {
+                cp_msg(loc("Not sending test report - "  .
+                        "bogus missing prerequisites report"));
+                return 1;
+            }
+            $message .= REPORT_MISSING_PREREQS->($author,$email,@missing);
         }
 
         ### was it missing test files? ###
         if( NO_TESTS_DEFINED->($buffer) ) {
             $message .= REPORT_MISSING_TESTS->();
         }
-    }
+
+        ### add a list of what modules have been loaded of your prereqs list
+        $message .= REPORT_LOADED_PREREQS->($mod);
+
+        ### the footer
+        $message .=  REPORT_MESSAGE_FOOTER->();
+
+    ### it may be another grade than fail/unknown.. may be worth noting
+    ### that tests got skipped, since the buffer is not added in
+    } elsif ( $tests_skipped ) {
+        $message .= REPORT_TESTS_SKIPPED->();
+    }        
 
     ### if it failed, and that already got reported, we're not cc'ing the
     ### author. Also, 'dont_cc' might be in the config, so check this;
@@ -364,7 +412,7 @@ sub _send_report {
             }
 
             if( $count > MAX_REPORT_SEND and !$force) {
-                msg(loc("'%1' already reported for '%2', ".
+                cp_msg(loc("'%1' already reported for '%2', ".
                         "not cc-ing the author",
                         GRADE_FAIL, $dist ), $verbose );
                 $dont_cc_author++;
@@ -383,12 +431,15 @@ sub _send_report {
     $reporter->from( $conf->get_conf('email') )
         if $conf->get_conf('email') !~ /\@example\.\w+$/i;
 
+    ### give the user a chance to programattically alter the message
+    $message = $self->_callbacks->munge_test_report->($mod, $message, $grade);
+
     ### add the body if we have any ###
     $reporter->comments( $message ) if defined $message && length $message;
 
     ### do a callback to ask if we should send the report
     unless ($self->_callbacks->send_test_report->($mod, $grade)) {
-        msg(loc("Ok, not sending test report"));
+        cp_msg(loc("Ok, not sending test report"));
         return 1;
     }
 
@@ -413,29 +464,75 @@ sub _send_report {
     ### should we save it locally? ###
     if( $save ) {
         if( my $file = $reporter->write() ) {
-            msg(loc("Successfully wrote report for '%1' to '%2'",
+            cp_msg(loc("Successfully wrote report for '%1' to '%2'",
                     $dist, $file), $verbose);
             return $file;
 
         } else {
-            error(loc("Failed to write report for '%1'", $dist));
+            cp_error(loc("Failed to write report for '%1'", $dist));
             return;
         }
 
     ### should we send it to a bunch of people? ###
     ### XXX should we do an 'already sent' check? ###
     } elsif( $reporter->send( @inform ) ) {
-        msg(loc("Successfully sent report for '%1'", $dist), $verbose);
+        cp_msg(loc("Successfully sent '%1' report for '%2'", $grade, $dist),
+            $verbose);
         return 1;
 
     ### something broke :( ###
     } else {
-        error(loc("Could not send report for '%1': %2",
-                $dist, $reporter->errstr));
+        cp_error(loc("Could not send '%1' report for '%2': %3",
+                $grade, $dist, $reporter->errstr));
         return;
     }
 }
 
+sub _verify_missing_prereqs {
+    my $self = shift;
+    my %hash = @_;
+
+    ### check arguments ###
+    my ($mod, $missing);
+    my $tmpl = {
+            module  => { required => 1, store => \$mod },
+            missing => { required => 1, store => \$missing },
+    };
+
+    check( $tmpl, \%hash ) or return;
+
+    
+    my %missing = map {$_ => 1} @$missing;
+    my $conf = $self->configure_object;
+    my $extract = $mod->status->extract;
+
+    ### Read pre-requisites from Makefile.PL or Build.PL (if there is one),
+    ### of the form:
+    ###     'PREREQ_PM' => {
+    ###                      'Compress::Zlib'        => '1.20',
+    ###                      'Test::More'            => 0,
+    ###                    },
+    ###  Build.PL uses 'requires' instead of 'PREREQ_PM'.
+
+    my @search;
+    push @search, ($extract ? MAKEFILE_PL->( $extract ) : MAKEFILE_PL->());
+    push @search, ($extract ? BUILD_PL->( $extract ) : BUILD_PL->());
+
+    for my $file ( @search ) {
+        if(-e $file and -r $file) {
+            my $slurp = $self->_get_file_contents(file => $file);
+            my ($prereq) = 
+                ($slurp =~ /'?(?:PREREQ_PM|requires)'?\s*=>\s*{(.*?)}/s);
+            my @prereq = 
+                ($prereq =~ /'?([\w\:]+)'?\s*=>\s*'?\d[\d\.\-\_]*'?/sg);
+            delete $missing{$_} for(@prereq);
+        }
+    }
+
+    return 1    if(keys %missing);  # There ARE missing prerequisites
+    return;                         # All prerequisites accounted for
+}
+
 1;
 
 
