@@ -251,7 +251,7 @@ The checksum value this distribution is expected to have
 
 =head1 METHODS
 
-=head2 new( OPTIONS )
+=head2 $self = CPANPLUS::Module::new( OPTIONS )
 
 This method returns a C<CPANPLUS::Module> object. Normal users
 should never call this method directly, but instead use the
@@ -288,8 +288,8 @@ sub new {
     my $acc = Object::Accessor->new;
     $acc->mk_accessors( qw[ installer_type dist_cpan dist prereqs
                             signature extract fetch readme uninstall
-                            created installed checksums checksum_ok
-                            checksum_value ] );
+                            created installed prepared checksums 
+                            checksum_ok checksum_value _fetch_from] );
 
     $object->status( $acc );
 
@@ -325,7 +325,7 @@ would be C<tar.gz>.
 Returns a boolean indicating of the package a particular module is in,
 is actually a core perl distribution.
 
-=head2 $mod->module_is_supplied_with_perl_core
+=head2 $mod->module_is_supplied_with_perl_core( [version => $]] )
 
 Returns a boolean indicating whether C<ANY VERSION> of this module
 was supplied with the current running perl's core package.
@@ -379,10 +379,11 @@ with C<Bundle::>.
 
     sub module_is_supplied_with_perl_core {
         my $self = shift;
+        my $ver  = shift || $];
 
         ### check Module::CoreList to see if it's a core package
         require Module::CoreList;
-        my $core = $Module::CoreList::version{ $] }->{ $self->module };
+        my $core = $Module::CoreList::version{ $ver }->{ $self->module };
 
         return $core;
     }
@@ -394,7 +395,7 @@ with C<Bundle::>.
 
 =pod
 
-=head2 clone
+=head2 $clone = $self->clone
 
 Clones the current module object for tinkering with.
 It will have a clean C<CPANPLUS::Module::Status> object, as well as
@@ -418,7 +419,7 @@ sub clone {
 
 =pod
 
-=head2 fetch
+=head2 $where = $self->fetch
 
 Fetches the module from a CPAN mirror.
 Look at L<CPANPLUS::Internals::Fetch::_fetch()> for details on the
@@ -430,14 +431,22 @@ sub fetch {
     my $self = shift;
     my $cb   = $self->parent;
 
-    my $where = $cb->_fetch( @_, module => $self ) or return;
+    ### custom args
+    my %args            = ( module => $self );
+
+    ### if a custom fetch location got specified before, add that here
+    $args{fetch_from}   = $self->status->_fetch_from 
+                            if $self->status->_fetch_from;
+
+    my $where = $cb->_fetch( @_, %args ) or return;
 
     ### do an md5 check ###
-    if( $cb->configure_object->get_conf('md5') and
+    if( !$self->status->_fetch_from and 
+        $cb->configure_object->get_conf('md5') and
         $self->package ne CHECKSUMS
     ) {
         unless( $self->_validate_checksum ) {
-            error( loc( "Checksum error for '%1' -- will not trust package",
+            cp_error( loc( "Checksum error for '%1' -- will not trust package",
                         $self->package) );
             return;
         }
@@ -448,7 +457,7 @@ sub fetch {
 
 =pod
 
-=head2 extract
+=head2 $path = $self->extract
 
 Extracts the fetched module.
 Look at L<CPANPLUS::Internals::Extract::_extract()> for details on
@@ -461,7 +470,7 @@ sub extract {
     my $cb   = $self->parent;
 
     unless( $self->status->fetch ) {
-        error( loc( "You have not fetched '%1' yet -- cannot extract",
+        cp_error( loc( "You have not fetched '%1' yet -- cannot extract",
                     $self->module) );
         return;
     }
@@ -469,7 +478,7 @@ sub extract {
     return $cb->_extract( @_, module => $self );
 }
 
-=head2 get_installer_type([prefer_makefile => BOOL])
+=head2 $type = $self->get_installer_type([prefer_makefile => BOOL])
 
 Gets the installer type for this module. This may either be C<build> or
 C<makemaker>. If C<Module::Build> is unavailable or no installer type
@@ -497,7 +506,7 @@ sub get_installer_type {
 
     my $extract = $self->status->extract();
     unless( $extract ) {
-        error(loc("Cannot determine installer type of unextracted module '%1'",
+        cp_error(loc("Cannot determine installer type of unextracted module '%1'",
                   $self->module));
         return;
     }
@@ -514,16 +523,18 @@ sub get_installer_type {
     $type = INSTALLER_MM    if  $found_makefile  && !$found_build;
 
     ### ok, so it's a 'build' installer, but you don't /have/ module build
-    unless( check_install( module => 'Module::Build' ) ) {
-        error( loc( "This module requires '%1' to be installed, ".
+    if( $type eq INSTALLER_BUILD and ( 
+            not grep { $_ eq INSTALLER_BUILD } CPANPLUS::Dist->dist_types )
+    ) {
+        cp_error( loc( "This module requires '%1' and '%2' to be installed, ".
                     "but you don't have it! Will fall back to ".
-                    "'%2', but might not be able to install!",
-                     'Module::Build', INSTALLER_MM ) );
+                    "'%3', but might not be able to install!",
+                     'Module::Build', INSTALLER_BUILD, INSTALLER_MM ) );
         $type = INSTALLER_MM;
 
     ### ok, actually we found neither ###
     } elsif ( !$type ) {
-        error( loc( "Unable to find '%1' or '%2' for '%3'; ".
+        cp_error( loc( "Unable to find '%1' or '%2' for '%3'; ".
                     "Will default to '%4' but might be unable ".
                     "to install!", BUILD_PL->(), MAKEFILE_PL->(),
                     $self->module, INSTALLER_MM ) );
@@ -536,7 +547,7 @@ sub get_installer_type {
 
 =pod
 
-=head2 dist([format => DISTRIBUTION_TYPE, args => {key => val}]);
+=head2 $dist = $self->dist([target => 'prepare|create', format => DISTRIBUTION_TYPE, args => {key => val}]);
 
 Create a distribution object, ready to be installed.
 Distribution type defaults to your config settings
@@ -556,11 +567,12 @@ sub dist {
     my $conf = $cb->configure_object;
     my %hash = @_;
 
-    my $type; my $args;
+    my($type,$args,$target);
     my $tmpl = {
         format  => { default => $conf->get_conf('dist_type') ||
                                 $self->status->installer_type,
                      store   => \$type },
+        target  => { default => TARGET_CREATE, store => \$target },                     
         args    => { default => {}, store => \$args },
     };
 
@@ -570,18 +582,54 @@ sub dist {
                                     module => $self
                             ) or return;
 
-    $dist->create( %$args ) or return;
+    DIST: {
+        ### first prepare the dist
+        $dist->prepare( %$args ) or return;
+        $self->status->prepared(1);
 
-    $self->status->created(1);
+        ### you just wanted us to prepare?
+        last DIST if $target eq TARGET_PREPARE;
 
-    #$self->status->dist( $dist );
+        $dist->create( %$args ) or return;
+        $self->status->created(1);
+    }
 
     return $dist;
 }
 
 =pod
 
-=head2 $mod->test( )
+=head2 $bool = $mod->prepare( )
+ 
+Convenience method around C<install()> that prepares a module 
+without actually building it. This is equivalent to invoking C<install>
+with C<target> set to C<prepare>
+
+Returns true on success, false on failure.
+
+=cut
+
+sub prepare { 
+    my $self = shift;
+    return $self->install( @_, target => TARGET_PREPARE );
+}
+
+=head2 $bool = $mod->create( )
+
+Convenience method around C<install()> that creates a module. 
+This is equivalent to invoking C<install> with C<target> set to 
+C<create>
+
+Returns true on success, false on failure.
+
+=cut
+
+sub create { 
+    my $self = shift;
+    return $self->install( @_, target => TARGET_CREATE );
+}
+
+=head2 $bool = $mod->test( )
 
 Convenience wrapper around C<install()> that tests a module, without
 installing it.
@@ -594,12 +642,12 @@ Returns true on success, false on failure.
 
 sub test {
     my $self = shift;
-    return $self->install( @_, target => 'create', skiptest => 0 );
+    return $self->install( @_, target => TARGET_CREATE, skiptest => 0 );
 }
 
 =pod
 
-=head2 install([ target => 'create|install', format => FORMAT_TYPE, extractdir => DIRECTORY, fetchdir => DIRECTORY, prefer_bin => BOOL, force => BOOL, verbose => BOOL, ..... ]);
+=head2 $bool = $self->install([ target => 'prepare|create|install', format => FORMAT_TYPE, extractdir => DIRECTORY, fetchdir => DIRECTORY, prefer_bin => BOOL, force => BOOL, verbose => BOOL, ..... ]);
 
 Installs the current module. This includes fetching it and extracting
 it, if this hasn't been done yet, as well as creating a distribution
@@ -629,8 +677,9 @@ sub install {
         ### targets 'dist' and 'test' are now completely ignored ###
         my $tmpl = {
                         ### match this allow list with Dist->_resolve_prereqs
-            target     => { default => 'install', store => \$target,
-                                allow => [qw|create install|] },
+            target     => { default => TARGET_INSTALL, store => \$target,
+                            allow   => [TARGET_PREPARE, TARGET_CREATE,
+                                        TARGET_INSTALL] },
             force      => { default => $conf->get_conf('force'), },
             verbose    => { default => $conf->get_conf('verbose'), },
             format     => { default => $conf->get_conf('dist_type'),
@@ -640,16 +689,19 @@ sub install {
         $args = check( $tmpl, \%hash ) or return;
     }
 
-    ### if this target is 'create' then so is the target of every prereq
-    $args->{'prereq_target'} ||= 'create' if $target eq 'create';
+    ### if this target isn't 'install', we will need to at least 'create' 
+    ### every prereq, so it can build
+    ### XXX prereq_target of 'prepare' will do weird things here, and is
+    ### not supported.
+    $args->{'prereq_target'} ||= TARGET_CREATE if $target ne TARGET_INSTALL;
 
     ### check if it's already upto date ###
-    if( $target eq 'install' and !$args->{'force'} and
+    if( $target eq TARGET_INSTALL and !$args->{'force'} and
         !$self->package_is_perl_core() and         # separate rules apply
         ( $self->status->installed() or $self->is_uptodate ) and
         !INSTALL_VIA_PACKAGE_MANAGER->($format)
     ) {
-        msg(loc("Module '%1' already up to date, won't install without force",
+        cp_msg(loc("Module '%1' already up to date, won't install without force",
                 $self->module), $args->{'verbose'} );
         return $self->status->installed(1);
     }
@@ -658,20 +710,20 @@ sub install {
     if( $self->package_is_perl_core() ) {
         # if the installed is newer, say so.
         if( $self->installed_version > $self->version ) {
-            error(loc("The core Perl %1 module '%2' (%3) is more ".
+            cp_error(loc("The core Perl %1 module '%2' (%3) is more ".
                       "recent than the latest release on CPAN (%4). ".
                       "Aborting install.",
                       $], $self->module, $self->installed_version,
                       $self->version ) );
         # if the installed matches, say so.
         } elsif( $self->installed_version == $self->version ) {
-            error(loc("The core Perl %1 module '%2' (%3) can only ".
+            cp_error(loc("The core Perl %1 module '%2' (%3) can only ".
                       "be installed by Perl itself. ".
                       "Aborting install.",
                       $], $self->module, $self->installed_version ) );
         # otherwise, the installed is older; say so.
         } else {
-            error(loc("The core Perl %1 module '%2' can only be ".
+            cp_error(loc("The core Perl %1 module '%2' can only be ".
                       "upgraded from %3 to %4 by Perl itself (%5). ".
                       "Aborting install.",
                       $], $self->module, $self->installed_version,
@@ -707,7 +759,7 @@ sub install {
     $format ||= $self->status->installer_type;
 
     unless( $format ) {
-        error( loc( "Don't know what installer to use; " .
+        cp_error( loc( "Don't know what installer to use; " .
                     "Couldn't find either '%1' or '%2' in the extraction " .
                     "directory '%3' -- will be unable to install",
                     BUILD_PL->(), MAKEFILE_PL->(), $self->status->extract ) );
@@ -720,7 +772,7 @@ sub install {
     ### do SIGNATURE checks? ###
     if( $conf->get_conf('signature') ) {
         unless( $self->check_signature( verbose => $args->{verbose} ) ) {
-            error( loc( "Signature check failed for module '%1' ".
+            cp_error( loc( "Signature check failed for module '%1' ".
                         "-- Not trusting this module, aborting install",
                         $self->module ) );
             $self->status->signature(0);
@@ -741,21 +793,23 @@ sub install {
         ### check what we need to install ###
         my @prereqs = $self->bundle_modules();
         unless( @prereqs ) {
-            error( loc( "Bundle '%1' does not specify any modules to install",
+            cp_error( loc( "Bundle '%1' does not specify any modules to install",
                         $self->module ) );
 
             ### XXX mark an error here? ###
         }
     }
 
-    my $dist = $self->dist( format => $format, args => $args );
+    my $dist = $self->dist( format  => $format, 
+                            target  => $target, 
+                            args    => $args );
     unless( $dist ) {
-        error( loc( "Unable to create a new distribution object for '%1' " .
+        cp_error( loc( "Unable to create a new distribution object for '%1' " .
                     "-- cannot continue", $self->module ) );
         return;
     }
 
-    return 1 if $target ne 'install';
+    return 1 if $target ne TARGET_INSTALL;
 
     my $ok = $dist->install( %$args ) ? 1 : 0;
 
@@ -765,7 +819,7 @@ sub install {
     return;
 }
 
-=pod bundle_modules()
+=pod @list = $self->bundle_modules()
 
 Returns a list of module objects the Bundle specifies.
 
@@ -781,13 +835,13 @@ sub bundle_modules {
     my $cb   = $self->parent;
 
     unless( $self->is_bundle ) {
-        error( loc("'%1' is not a bundle", $self->module ) );
+        cp_error( loc("'%1' is not a bundle", $self->module ) );
         return;
     }
 
     my $dir;
     unless( $dir = $self->status->extract ) {
-        error( loc("Don't know where '%1' was extracted to", $self->module ) );
+        cp_error( loc("Don't know where '%1' was extracted to", $self->module ) );
         return;
     }
 
@@ -800,7 +854,7 @@ sub bundle_modules {
     my $prereqs = {}; my @list; my $seen = {};
     for my $file ( @files ) {
         my $fh = FileHandle->new($file)
-                    or( error(loc("Could not open '%1' for reading: %2",
+                    or( cp_error(loc("Could not open '%1' for reading: %2",
                         $file,$!)), next );
 
         my $flag;
@@ -821,7 +875,7 @@ sub bundle_modules {
                 my $obj = $cb->module_tree($module);
 
                 unless( $obj ) {
-                    error(loc("Cannot find bundled module '%1'", $module),
+                    cp_error(loc("Cannot find bundled module '%1'", $module),
                           loc("-- it does not seem to exist") );
                     next;
                 }
@@ -844,7 +898,7 @@ sub bundle_modules {
 
 =pod
 
-=head2 readme
+=head2 $text = $self->readme
 
 Fetches the readme belonging to this module and stores it under
 C<< $obj->status->readme >>. Returns the readme as a string on
@@ -875,7 +929,7 @@ sub readme {
     ### read the file into a scalar, to store in the original object ###
     my $fh = new FileHandle;
     unless( $fh->open($file) ) {
-        error( loc( "Could not open file '%1': %2", $file, $! ) );
+        cp_error( loc( "Could not open file '%1': %2", $file, $! ) );
         return;
     }
 
@@ -888,16 +942,16 @@ sub readme {
 
 =pod
 
-=head2 installed_version()
+=head2 $version = $self->installed_version()
 
 Returns the currently installed version of this module, if any.
 
-=head2 installed_file()
+=head2 $where = $self->installed_file()
 
 Returns the location of the currently installed file of this module,
 if any.
 
-=head2 is_uptodate([version => VERSION_NUMBER])
+=head2 $bool = $self->is_uptodate([version => VERSION_NUMBER])
 
 Returns a boolean indicating if this module is uptodate or not.
 
@@ -937,7 +991,7 @@ Returns a boolean indicating if this module is uptodate or not.
 
 =pod
 
-=head2 details()
+=head2 $href = $self->details()
 
 Returns a hashref with key/value pairs offering more information about
 a particular module. For example, for C<Time::HiRes> it might look like
@@ -984,17 +1038,39 @@ sub details {
     return $res;
 }
 
+=head2 @list = $self->contains()
+
+Returns a list of module objects that represent the modules also 
+present in the package of this module.
+
+For example, for C<Archive::Tar> this might return:
+
+    Archive::Tar
+    Archive::Tar::Constant
+    Archive::Tar::File
+
+=cut
+
+sub contains {
+    my $self = shift;
+    my $cb   = $self->parent;
+    my $pkg  = $self->package;
+    
+    my @mods = $cb->search( type => 'package', allow => [qr/^$pkg$/] );
+    
+    return @mods;
+}
+
 =pod
 
-=head2 fetch_report()
+=head2 @list_of_hrefs = $self->fetch_report()
 
 This function queries the CPAN testers database at
 I<http://testers.cpan.org/> for test results of specified module
 objects, module names or distributions.
 
 Look at L<CPANPLUS::Internals::Report::_query_report()> for details on
-the options you can pass.
-
+the options you can pass and the return value to expect.
 
 =cut
 
@@ -1007,7 +1083,7 @@ sub fetch_report {
 
 =pod
 
-=head2 uninstall([type => [all|man|prog])
+=head2 $bool = $self->uninstall([type => [all|man|prog])
 
 This function uninstalls the specified module object.
 
@@ -1047,7 +1123,7 @@ sub uninstall {
         ($conf->get_conf('dist_type') ne INSTALLER_BUILD) or
         ($conf->get_conf('dist_type') ne INSTALLER_MM))
     ) {
-        msg(loc("You have a default installer type set (%1) ".
+        cp_msg(loc("You have a default installer type set (%1) ".
                 "-- you should probably use that package manager to " .
                 "uninstall modules", $conf->get_conf('dist_type')), $verbose);
     }
@@ -1055,7 +1131,7 @@ sub uninstall {
     ### check if we even have the module installed -- no point in continuing
     ### otherwise
     unless( $self->installed_version ) {
-        error( loc( "Module '%1' is not installed, so cannot uninstall",
+        cp_error( loc( "Module '%1' is not installed, so cannot uninstall",
                     $self->module ) );
         return;
     }
@@ -1074,14 +1150,14 @@ sub uninstall {
     for my $file( @$files, $pack ) {
         next unless defined $file && -f $file;
 
-        msg(loc("Unlinking '%1'", $file), $verbose);
+        cp_msg(loc("Unlinking '%1'", $file), $verbose);
 
         my $buffer;
         unless ( run(   command => [$sudo, $^X, "-eunlink+q[$file]"],
                         verbose => $verbose,
                         buffer  => \$buffer )
         ) {
-            error(loc("Failed to unlink '%1': '%2'",$file, $buffer));
+            cp_error(loc("Failed to unlink '%1': '%2'",$file, $buffer));
             $flag++;
         }
     }
@@ -1094,13 +1170,13 @@ sub uninstall {
 
         next unless @count == 2;    # . and ..
 
-        msg(loc("Removing '%1'", $dir), $verbose);
+        cp_msg(loc("Removing '%1'", $dir), $verbose);
 
         ### this fails on my win2k machines.. it indeed leaves the
         ### dir, but it's not a critical error, since the files have
         ### been removed. --kane
         #unless( rmdir $dir ) {
-        #    error( loc( "Could not remove '%1': %2", $dir, $! ) )
+        #    cp_error( loc( "Could not remove '%1': %2", $dir, $! ) )
         #        unless $^O eq 'MSWin32';
         #}
         my $buffer;
@@ -1108,7 +1184,7 @@ sub uninstall {
                         verbose => $verbose,
                         buffer  => \$buffer )
         ) {
-            error(loc("Failed to rmdir '%1': %2",$dir,$buffer));
+            cp_error(loc("Failed to rmdir '%1': %2",$dir,$buffer));
             $flag++;
         }
     }
@@ -1121,7 +1197,7 @@ sub uninstall {
 
 =pod
 
-=head2 distributions()
+=head2 @modobj = $self->distributions()
 
 Returns a list of module objects representing all releases for this
 module on success, false on failure.
@@ -1140,7 +1216,7 @@ sub distributions {
 
 =pod
 
-=head2 files ()
+=head2 @list = $self->files ()
 
 Returns a list of files used by this module, if it is installed.
 
@@ -1152,7 +1228,7 @@ sub files {
 
 =pod
 
-=head2 directory_tree ()
+=head2 @list = $self->directory_tree ()
 
 Returns a list of directories used by this module.
 
@@ -1164,7 +1240,7 @@ sub directory_tree {
 
 =pod
 
-=head2 packlist ()
+=head2 @list = $self->packlist ()
 
 Returns the C<ExtUtils::Packlist> object for this module.
 
@@ -1176,7 +1252,7 @@ sub packlist {
 
 =pod
 
-=head2 validate ()
+=head2 @list = $self->validate ()
 
 Returns a list of files that are missing for this modules, but
 are present in the .packlist file.
@@ -1212,7 +1288,7 @@ sub _extutils_installed {
     ### old versions of cygwin + perl < 5.8 are buggy here. bail out if we
     ### find we're being used by them
     {   my $err = ON_OLD_CYGWIN;
-        if($err) { error($err); return };
+        if($err) { cp_error($err); return };
     }
 
     return unless can_load(
@@ -1222,7 +1298,7 @@ sub _extutils_installed {
 
     my $inst;
     unless( $inst = ExtUtils::Installed->new() ) {
-        error( loc("Could not create an '%1' object", 'ExtUtils::Installed' ) );
+        cp_error( loc("Could not create an '%1' object", 'ExtUtils::Installed' ) );
 
         ### in case it's being used directly... ###
         return;
@@ -1235,7 +1311,7 @@ sub _extutils_installed {
 
         if( $@ ) {
             chomp $@;
-            error( loc("Could not get '%1' for '%2': %3",
+            cp_error( loc("Could not get '%1' for '%2': %3",
                         $method, $self->module, $@ ) );
             return;
         }
