@@ -2,12 +2,15 @@ package CPANPLUS::Module::Author::Fake;
 
 use CPANPLUS::inc;
 use CPANPLUS::Module::Author;
-@ISA = qw[CPANPLUS::Module::Author];
+use CPANPLUS::Internals;
+use CPANPLUS::Error;
 
 use strict;
 use vars            qw[@ISA];
 use Params::Check   qw[check];
 
+@ISA = qw[CPANPLUS::Module::Author];
+
 $Params::Check::VERBOSE = 1;
 
 =pod
@@ -50,13 +53,18 @@ sub new {
         author  => { default    => 'CPANPLUS Internals' },
         email   => { default    => 'cpanplus-info@lists.sf.net' },
         cpanid  => { default    => 'CPANPLUS' },
-        _id     => { required   => 1 },
+        _id     => { default => CPANPLUS::Internals->_last_id },
     };
 
     my $args = check( $tmpl, \%hash ) or return;
 
     my $obj = CPANPLUS::Module::Author->new( %$args ) or return;
 
+    unless( $obj->_id ) {
+        cp_error(loc("No '%1' specified -- No CPANPLUS object associated!",'_id'));
+        return;
+    } 
+
     ### rebless object ###
     return bless $obj, $class;
 }
