@@ -53,7 +53,9 @@ choice.
 sub import {
     my $class   = shift;
     my $option  = shift;
-    my $conf    = CPANPLUS::Configure->new();
+    ### XXX this should offer to reconfigure CPANPLUS, somehow.  --rs
+    my $conf    = CPANPLUS::Configure->new() 
+                    or die loc("No configuration available -- aborting") . $/;
 
     ### find out what shell we're supposed to load ###
     $SHELL      = $option
@@ -211,7 +213,7 @@ sub _pager_open {
 
     my $fh = new FileHandle;
     unless ( $fh->open("| $cmd") ) {
-        error(loc("could not pipe to %1: %2\n", $cmd, $!) );
+        cp_error(loc("could not pipe to %1: %2\n", $cmd, $!) );
         return;
     }
 
