@@ -25,7 +25,7 @@ local $Params::Check::VERBOSE = 1;
 BEGIN {
     use vars        qw[ $VERSION @ISA ];
     @ISA        =   qw[ CPANPLUS::Shell::_Base::ReadLine ];
-    $VERSION    =   '0.050_04';
+    $VERSION    =   '0.056_01';
 }
 
 load CPANPLUS::Shell;
@@ -101,6 +101,7 @@ CPANPLUS::Shell::Default
 
     cpanp> i Acme::Foo      # install Acme::Foo
     cpanp> i Acme-Foo-1.3   # install version 1.3 of Acme::Foo
+    cpanp> i <URI>          # install from URI, like http://foo.com/X.tgz
     cpanp> i 1 3..5         # install search results 1, 3, 4 and 5
     cpanp> i *              # install all search results
     cpanp> a KANE; i *;     # find modules by kane, install all results
@@ -115,7 +116,8 @@ CPANPLUS::Shell::Default
     cpanp> r Acme::Foo      # view Acme::Foo's README file
     cpanp> o                # get a list of all installed modules that
                             # are out of date
-
+    cpanp> o 1..3           # list uptodateness from a previous search 
+                            
     cpanp> s conf           # show config settings
     cpanp> s conf md5 1     # enable md5 checks
     cpanp> s program        # show program settings
@@ -128,7 +130,7 @@ CPANPLUS::Shell::Default
 
     cpanp> b                # create an autobundle for this computers
                             # perl installation
-    cpanp> x                # reload index files
+    cpanp> x                # reload index files (purges cache)
     cpanp> p [FILE]         # print error stack (to a file)
     cpanp> v                # show the banner
     cpanp> w                # show last search results again
@@ -217,10 +219,11 @@ sub _input_loop {
             $SIG{$sig} = $entry->{handler} if exists($entry->{handler});
         }
 
+	print "\n";
         last if $self->dispatch_on_input( input => $input );
 
         ### flush the lib cache ###
-        $cb->_flush( list => [qw|lib|] );
+        $cb->_flush( list => [qw|lib load|] );
 
     } continue {
         $self->_signals->{INT}{count}--
@@ -474,6 +477,7 @@ loc("    o [ MODULE ... ]       # list installed module(s) that aren't up to dat
 loc('    w                      # display the result of your last search again'     ),
 loc('[Operations]'                                                                  ),
 loc('    i MODULE | NUMBER ...  # install module(s), by name or by search number'   ),
+loc('    i URI | ...            # install module(s), by URI (ie http://foo.com/X.tgz)'   ),
 loc('    t MODULE | NUMBER ...  # test module(s), by name or by search number'      ),
 loc('    u MODULE | NUMBER ...  # uninstall module(s), by name or by search number' ),
 loc('    d MODULE | NUMBER ...  # download module(s)'                               ),
@@ -491,7 +495,7 @@ loc('    s edit                 # open configuration file in editor and reload'
 loc('    /source FILE [FILE ..] # read in commands from the specified file'         ),
 loc('    ! EXPR                 # evaluate a perl statement'                        ),
 loc('    p [FILE]               # print the error stack (optionally to a file)'     ),
-loc('    x                      # reload CPAN indices'                              ),
+loc('    x                      # reload CPAN indices (purges cache)'                              ),
     ) unless @Help;
 
     $self->_pager_open if (@Help >= $self->_term_rowcount);
@@ -520,7 +524,7 @@ sub _bang {
     }
 
     eval $input;
-    error( $@ ) if $@;
+    cp_error( $@ ) if $@;
     print "\n";
     return;
 }
@@ -776,8 +780,9 @@ sub _install {
         return;
     }
 
-    my $target = $choice eq 'i' ? 'install' : 'create';
+    my $target = $choice eq 'i' ? TARGET_INSTALL : TARGET_CREATE;
     my $prompt = $choice eq 'i' ? loc('Installing ') : loc('Testing ');
+    my $action = $choice eq 'i' ? 'install' : 'test';
 
     my $status = {};
     ### first loop over the mods to install them ###
@@ -795,20 +800,20 @@ sub _install {
     #    if( $mod->status->installed ) {
         if( $status->{$mod} ) {
             print loc("Module '%1' %tense(%2,past) successfully\n",
-                        $mod->module, $target)
+                        $mod->module, $action)
         } else {
             $flag++;
             print loc("Error %tense(%1,present) '%2'\n",
-                        $target, $mod->module);
+                        $action, $mod->module);
         }
     }
 
 
 
     if( !$flag ) {
-        print loc("No errors %tense(%1,present) all modules", $target), "\n";
+        print loc("No errors %tense(%1,present) all modules", $action), "\n";
     } else {
-        print loc("Problem %tense(%1,present) one or more modules", $target);
+        print loc("Problem %tense(%1,present) one or more modules", $action);
         print "\n";
         print loc("*** You can view the complete error buffer by pressing '%1' ***\n", 'p')
                 unless $conf->get_conf('verbose') || $self->noninteractive;
@@ -848,7 +853,7 @@ sub __ask_about_send_test_report {
     my $term    = $Shell->term;
 
     print "\n";
-    print loc(  "Test report prepared for module '%1'\n. Would you like to ".
+    print loc(  "Test report prepared for module '%1'.\n Would you like to ".
                 "send it? (You can edit it if you like)", $mod->module );
     print "\n\n";
     my $bool =  $term->ask_yn(
@@ -900,8 +905,11 @@ sub _details {
     ### maybe more later with Module::CPANTS etc
     $self->_pager_open if scalar @$mods * 10 > $self->_term_rowcount;
 
+
+    my $format = "%-30s %-30s\n";
     for my $mod (@$mods) {
         my $href = $mod->details( %$opts );
+        my @list = sort { $a->module cmp $b->module } $mod->contains;
 
         unless( $href ) {
             print loc("No details for %1 - it might be outdated.",
@@ -911,7 +919,13 @@ sub _details {
         } else {
             print loc( "Details for '%1'\n", $mod->module );
             for my $item ( sort keys %$href ) {
-                printf "%-30s %-30s\n", $item, $href->{$item};
+                printf $format, $item, $href->{$item};
+            }
+            
+            my $showed;
+            for my $item ( @list ) {
+                printf $format, ($showed ? '' : 'Contains:'), $item->module;
+                $showed++;
             }
             print "\n";
         }
@@ -967,7 +981,7 @@ sub _set_conf {
 
     ### possible options
     ### XXX hard coded, not optimal :(
-    my @types   = qw[reconfigure save edit program conf];
+    my @types   = qw[reconfigure save edit program conf mirrors];
 
 
     my $args; my $opts; my $input;
@@ -986,9 +1000,9 @@ sub _set_conf {
 
     if( $type eq 'reconfigure' ) {
         my $setup = CPANPLUS::Configure::Setup->new(
-                        conf    => $conf,
-                        term    => $self->term,
-                        backend => $cb,
+                        configure_object    => $conf,
+                        term                => $self->term,
+                        backend             => $cb,
                     );
         return $setup->init;
 
@@ -1022,6 +1036,19 @@ sub _set_conf {
 
         return 1;
 
+    } elsif ( $type eq 'mirrors' ) {
+    
+        print loc("Readonly list of mirrors (in order of preference):\n\n" );
+        
+        my $i;
+        for my $host ( @{$conf->get_conf('hosts')} ) {
+            my $uri = $cb->_host_to_uri( %$host );
+            
+            $i++;
+            print "\t[$i] $uri\n";
+        }
+        
+
     } else {
 
         if ( $type eq 'program' or $type eq 'conf' ) {
@@ -1109,7 +1136,7 @@ sub _uptodate {
     for my $mod ( @rv ) {
         printf $format,
                 $i,
-                $self->_format_version( $mod->installed_version ),
+                $self->_format_version( $mod->installed_version ) || 'None',
                 $self->_format_version( $mod->version ),
                 $mod->module,
                 $mod->author->cpanid();
@@ -1308,7 +1335,7 @@ sub _meta {
                             PeerAddr    => $host,
                             PeerPort    => $port,
                         ) or (
-                            error( loc( "Cannot connect to port '%1' ".
+                            cp_error( loc( "Cannot connect to port '%1' ".
                                         "on host '%2'", $port, $host ) ),
                             return
                         );
@@ -1371,7 +1398,7 @@ sub _meta {
     } elsif ( $cmd eq 'source' ) {
         while( my $file = shift @parts ) {
             my $fh = FileHandle->new("$file")
-                        or( error(loc("Could not open file '%1': %2",
+                        or( cp_error(loc("Could not open file '%1': %2",
                             $file, $!)),
                             return
                         );
@@ -1381,7 +1408,7 @@ sub _meta {
             }
         }
     } else {
-        error( loc( "Unknown command '%1'", $cmd ) );
+        cp_error( loc( "Unknown command '%1'", $cmd ) );
         return;
     }
     return 1;
